import json
from django.conf import settings
from django.shortcuts import render
from django.http import JsonResponse,HttpResponseNotAllowed
from .thvisual import THVClient
# Create your views here.

def visualApps(request):
    return render(request,"thvapps.html",{})

def openApp(request):
    if request.method == "POST":
        pData = json.loads(request.body)
        vapp = pData.get('vapp')
        version = pData.get('version')
        gpu = pData.get('gpu')
        winWidth = pData.get('winWidth')
        winHeight = pData.get('winHeight','')

        if not winHeight == '':
            winWidth = int(winWidth * (winHeight - settings.TH_VISUALAPP_CONFIG.get("FOOTER_HEIGHT",80)) / winHeight)
            winHeight = int(winHeight - settings.TH_VISUALAPP_CONFIG.get("FOOTER_HEIGHT",80))
            winWidth = int(winHeight * settings.TH_VISUALAPP_CONFIG.get("SCREEN_RATIO"))
        print(winWidth)
        print(winHeight)
        server = settings.TH_VISUALAPP_CONFIG.get("VAPP_BACKEND_HOST")
        appid = settings.TH_VISUALAPP_CONFIG.get("VAPP_BACKEND_APPID")
        appkey = settings.TH_VISUALAPP_CONFIG.get("VAPP_BACKEND_APPKEY")

        cluster = request.session.get('cluster', '')
        user = request.session.get('systemUsername', '')

        client = THVClient(appid, appkey, server, cluster, user)
        try:
            res = client.launchApp(vapp, gpu, version, winWidth, winHeight)
            vncToken = res.get("token")
            wsURI = "ws://127.0.0.1/visual/v1/%s/%s/visual/%s/ws?token=%s" % (
                cluster, user, vapp, vncToken)
            vncPassword = res.get("vncPassword")
        except Exception as e:
            return JsonResponse({"success": "no", "error": e})
        return JsonResponse({"success": "yes", "wsURI": wsURI, 'vncYao': vncPassword})

def closeApp(request):
    if request.method == "POST":
        pData = json.loads(request.body)
        vapp = pData.get('vapp')
        version = pData.get('version')
        gpu = pData.get('gpu')

        server = settings.TH_VISUALAPP_CONFIG.get("VAPP_BACKEND_HOST")
        appid = settings.TH_VISUALAPP_CONFIG.get("VAPP_BACKEND_APPID")
        appkey = settings.TH_VISUALAPP_CONFIG.get("VAPP_BACKEND_APPKEY")

        cluster = request.session.get('cluster', '')
        user = request.session.get('systemUsername', '')

        client = THVClient(appid, appkey, server, cluster, user)
        try:
            client.closeApp(vapp, gpu, version)
        except Exception as e:
            return JsonResponse({"success": "no", "error": e})
        return JsonResponse({"success": "yes"})

def listApp(request):
    if not request.method == "POST":
        return HttpResponseNotAllowed("method not allowed")

    server = settings.TH_VISUALAPP_CONFIG.get("VAPP_BACKEND_HOST")
    appid = settings.TH_VISUALAPP_CONFIG.get("VAPP_BACKEND_APPID")
    appkey = settings.TH_VISUALAPP_CONFIG.get("VAPP_BACKEND_APPKEY")

    cluster = request.session.get('cluster', '')
    user = request.session.get('systemUsername', '')
    appIconsJson = request.session.get('vapp_icons')
    appIcons = json.loads(appIconsJson).get(cluster,{})

    client = THVClient(appid, appkey, server, cluster, user)
    try:
        res = client.listApps()
        apps = []
        appNames = list(set(list(map(lambda x: x.get("app"), res.get("vapps")))))
        defaultImage = settings.TH_VISUALAPP_CONFIG.get('VAPP_DEFAULT_IMGAGE_URL')
        for appName in appNames:
            newApp = {"name":"visual","app":appName}
            versions = []
            defaultVersion = ""
            defaultGPU = "no"
            gpus = []
            for a in res.get("vapps"):
                if a.get("app") == appName:
                    if a.get("version") not in versions:
                        versions.append(a.get("version"))
                    if a.get("default"):
                        defaultVersion = a.get('version')
                    if a.get("gpu",False):
                        gl = "GPU加速"
                        gk = "1"
                        gv = "yes"
                        defaultGPU = "yes"
                    else:
                        gl = "非GPU加速"
                        gk = "0"
                        gv = "no"
                    gpus.append({"key":gk,"val":gv,"lab":gl})
            newApp["versions"] = versions
            newApp["gpus"] = gpus
            newApp["defaultVersion"] = defaultVersion
            newApp["defaultGPU"] = defaultGPU
            newApp["value"] = appName
            apps.append(newApp)
            for key,appsAdv in appIcons.items():
                appIcons[key] = appsAdv
            if appIcons.get(appName,"") == "":
                newApp["image"] = defaultImage
            else:
                if appIcons.get(appName).get("video","") == "":
                    newApp["image"] = defaultImage
                else:
                    newApp["video"] = appIcons.get(appName).get("video")
                    newApp["showVideo"] = True
        apps.sort(key=lambda x:x.get("app"))
    except Exception as e:
        return JsonResponse({"success": "no", "error": e})
    return JsonResponse({"success":"yes","apps":apps})


#!/usr/bin/env python
# encoding: utf-8
# @author: Tianyang
# @license: (C) Copyright 2013-2020/4/20, NSCC-TJ.AllRightsReserved.
# @contact: tianyang@nscc-tj.cn
# @software: 
# @file: urls.py
# @time: 2020/4/20 18:43
# @desc:

from django.urls import path
import thvapp.views as thvapps

urlpatterns = [
    path('thva/apps', thvapps.visualApps),
    path('open/app', thvapps.openApp),
    path('close/app',thvapps.closeApp),
    path('list/app', thvapps.listApp),
]


import os
from setuptools import find_packages,setup

with open(os.path.join(os.path.dirname(__file__),'README.rst'),encoding='UTF-8') as readme:
    README = readme.read()

#allow setup.py to be run  from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__),os.pardir)))

setup(
    name='django-thvapp',
    version='1.0.8',
    packages=find_packages(),
    include_package_data=True,
    license='BSD License',
    description='A simple django app to use thuri web visual apps',
    long_description_content_type='text/x-rst',
    long_description=README,
    url='https://www.nscc-tj.cn',
    author='Tianyang',
    author_email='tianyang@nscc-tj.cn',
    classifiers=[
        'Framework :: Django',
        'Framework :: Django :: 2.0',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)


import hashlib
import time
import requests

class THVClient:
    def __init__(self, appid, appkey, server, cluster, user, timeout=60):
        self.appid = appid
        self.appkey = appkey
        self.server = server
        self.cluster = cluster
        self.user = user
        self.timeout = timeout

    def is_number(self,s):
        try:
            float(s)
            return True
        except ValueError:
            pass

        try:
            import unicodedata
            unicodedata.numeric(s)
            return True
        except (TypeError, ValueError):
            pass
        return False

    def _make_sig(self, timestamp, vapp=None):
        str1 = "appkey=%s&timestamp=%s&cluster=%s&user=%s" % (
            self.appkey, timestamp, self.cluster, self.user)

        if vapp is not None:
            str1 = str1 + "&vapp=" + vapp

        hashStr = hashlib.sha256()
        hashStr.update(str1.encode())
        sig = hashStr.hexdigest()
        return sig

    def listApps(self):
        url = "%s/v1/%s/%s/visual/list?appid=%s" % (self.server, self.cluster, self.user, self.appid)
        # print(url)
        timestamp = int(time.time())
        data = {
            "sig": self._make_sig(timestamp),
            "timestamp": timestamp,
        }

        resp = requests.post(url=url, data=data, timeout=self.timeout)

        if resp.status_code < 300:
            # print(resp.json())
            return resp.json()
        else:
            # print(resp.json())
            raise ValueError(resp.text)

    def launchApp(self, vapp,gpu=0,version="",winWidth=1920,winHeight=1200,param="",client="w"):
        if vapp == "shell":
            url = "%s/v1/%s/%s/web/shell/launch?appid=%s" % (self.server, self.cluster, self.user, self.appid)
            timestamp = int(time.time())
            data = {
                "sig": self._make_sig(timestamp),
                "timestamp": timestamp,
            }
        else:
            url = "%s/v1/%s/%s/visual/%s/launch?appid=%s" % (self.server, self.cluster, self.user, vapp, self.appid)
            # print(url)
            timestamp = int(time.time())
            data = {
                "sig": self._make_sig(timestamp, vapp),
                "timestamp": timestamp,
            }
            if param != "":
                data["param"] = param
            if int(gpu) == 1:
                data["gpu"] = "true"
            data["client"] = client
            data["version"] = version
            if not self.is_number(winWidth):
                winWidth = 1920
            if not self.is_number(winHeight):
                winHeight = 1200
            resolution = "{winw}x{winh}".format(winw=winWidth, winh=winHeight)
            data["resolution"] = resolution

        resp = requests.post(url=url, data=data, timeout=self.timeout)

        if resp.status_code < 300:
            resJ = resp.json()
            return resJ
        else:
            # print(resp.text)
            raise ValueError(resp.text)

    def closeApp(self,vapp,gpu=0,version="",param="",client="w"):
        if vapp == "shell":
            return {"status":False,"error":"shell app not allowed close!!"}
        else:
            url = "%s/v1/%s/%s/visual/%s/close?appid=%s" % (self.server, self.cluster, self.user, vapp, self.appid)
            timestamp = int(time.time())
            data = {
                "sig": self._make_sig(timestamp, vapp),
                "timestamp": timestamp,
            }
            if param != "":
                data["param"] = param
            if int(gpu) == 1:
                data["gpu"] = "true"
            data["client"] = client
            data["version"] = version
            # print(data)

        resp = requests.post(url=url, data=data, timeout=self.timeout)

        if resp.status_code < 300:
            return {"status":True}
        else:
            raise ValueError(resp.text)